package me.lensfrex.vegetables;

import me.lensfrex.vegetables.data.Vegetable;
import me.lensfrex.vegetables.exceptions.IllegalPageParamException;
import me.lensfrex.vegetables.exceptions.RequestPageTooLargeException;

import java.util.LinkedList;
import java.util.List;

/**
 * 请求处理工具
 * 由于功能较为简单，目前仅有获取分页数据的功能
 */
public class Tools {
    // 测试用路径，使用前记得改 :P
    public static final String SOURCE_FILE_PATH = "C:\\Users\\lenfrex\\projects\\AmberGround\\vegetables\\resources\\vegetables.txt";

    /**
     * 根据给定的页索引以及页大小，对全部蔬菜数据进行分页并返回分页后的List。
     *
     * @param pageNum          页码（从1开始）
     * @param pageSize         页大小
     * @param allVegetableList 全体的待分页的蔬菜数据
     * @return LinkedList<Vegetable> 分页后的蔬菜数据。
     * @throws RequestPageTooLargeException 如果请求的页码已经超过了最大页数
     * @throws IllegalPageParamException    如果分页请求的参数不对
     */
    public List<Vegetable> getListPage(int pageNum, int pageSize, List<Vegetable> allVegetableList)
            throws RequestPageTooLargeException, IllegalPageParamException {

        // 请求的分页大小或页码非法
        if (pageNum <= 0 || pageSize <= 0) {
            throw new IllegalPageParamException(
                    String.format("分页请求的参数不对：pageNum: %d, pageSize: %d", pageNum, pageSize));
        }

        int resultSize = allVegetableList.size();

        // 判断上一页的记录是不是已经超过最后一条记录，防止页码超过最大值
        if ((pageNum - 1)*pageSize >= resultSize) {
            throw new RequestPageTooLargeException();
        }

        LinkedList<Vegetable> pagedVegetableList = new LinkedList<>();
        for (int i = (pageNum - 1)*pageSize; (i < pageNum*pageSize && i < resultSize); i++) {
            pagedVegetableList.add(allVegetableList.get(i));
        }

        return pagedVegetableList;
    }

    /**
     * 根据数据字符串解析并获取蔬菜数据
     *
     * @param vegetableStrings 蔬菜数据源字符串列表，字符串信息格式应为
     *                         "蔬菜名 最低价 平均价 最高价 信息日期"
     *                         其中信息日期格式应为 "yyyy-mm-dd hh:mm:ss"
     * @return LinkedList<Vegetable> 解析后的蔬菜信息列表
     */
    public LinkedList<Vegetable> getAllVegetables(LinkedList<String> vegetableStrings) {
        LinkedList<Vegetable> vegetables = new LinkedList<>();
        for (String vegetableString : vegetableStrings) {
            vegetables.add(parseVegetableInfo(vegetableString));
        }

        return vegetables;
    }

    private static void printMessage() {
        System.out.println("Get data source from: " + SOURCE_FILE_PATH);
        System.out.println("This is just for test.");
        System.out.println("If some IO error occurred, try to change SOURCE_FILE_PATH in me.lensfrex.vegetables.Tools :P");
    }

    /**
     * 解析Vegetable的信息字符串，并返回相应的蔬菜信息数据
     *
     * @param vegetableString 蔬菜信息字符串，
     *                        格式应为 "蔬菜名 最低价 平均价 最高价 信息日期"
     *                        其中信息日期格式应为 "yyyy-mm-dd hh:mm:ss"
     * @return 蔬菜信息数据类
     */
    private Vegetable parseVegetableInfo(String vegetableString) {
        String[] spilt = vegetableString.split(" ");
        Vegetable vegetable = new Vegetable();

        vegetable.setName(spilt[0]);
        vegetable.setPrice(Double.parseDouble(spilt[1]), Vegetable.PRICE_MIN);
        vegetable.setPrice(Double.parseDouble(spilt[2]), Vegetable.PRICE_AVG);
        vegetable.setPrice(Double.parseDouble(spilt[3]), Vegetable.PRICE_MAX);
        vegetable.setPriceDate(spilt[4] + spilt[5]);

        return vegetable;
    }
}
